local L = LibStub("AceLocale-3.0"):GetLocale("Protwarbar")

function PWBInfo:OnInitialize()
	self.db = Protwarbar.db.profile.PWBInfo
	self.resolve_name = GetSpellInfo(158298)
	self.barrier = 0

	PWBInfo:BuildGui()

	self:RegisterMessage("PWB_ABILITIES_LOADED")
	self:RegisterMessage("PWB_PROFILE_CHANGED")
	self:RegisterMessage("PWB_REFRESH_CONFIG")
	self:RegisterMessage("PWB_UPDATE_VISIBILITY")
	self:RegisterMessage("PWB_UPDATE_HANDLERS")
end

function PWBInfo:OnEnable()
	PWBInfo:UpdateStats()
	PWBInfo:PWB_UPDATE_HANDLERS()
end

function PWBInfo:PWB_UPDATE_HANDLERS()
	self:CancelAllTimers()

	self:UnregisterEvent("UNIT_POWER_FREQUENT")
	self:UnregisterEvent("UNIT_ATTACK_POWER")
	self:UnregisterEvent("UNIT_STATS")

	if self.db.enabled and Protwarbar.db.profile.enabled then
		self:RegisterEvent("UNIT_POWER_FREQUENT")
		self:RegisterEvent("UNIT_ATTACK_POWER")
		self:RegisterEvent("UNIT_STATS")
		self:ScheduleRepeatingTimer("SlowTimer", 0.5)
	end
end

function PWBInfo:PWB_PROFILE_CHANGED()
	self.db = Protwarbar.db.profile.PWBInfo
end

function PWBInfo:PWB_ABILITIES_LOADED()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PWBInfo", PWBInfo:CreateOptions());
	Protwarbar.AceConfig:AddToBlizOptions("PWBInfo", L["Information"], "Protwarbar");
end

function PWBInfo:PWB_REFRESH_CONFIG()
	self:RefreshConfig()
end

function PWBInfo:PWB_UPDATE_VISIBILITY()
	self:UpdateVisiblity()
end

function PWBInfo:UNIT_POWER_FREQUENT(eventName, unit, powerType)
	if (unit == "player") then
		PWBInfo:Update(UnitPower("player"))
	end
end

function PWBInfo:UNIT_ATTACK_POWER(event, unit)
	if (unit == "player") then
		PWBInfo:UpdateStats()
	end
end

function PWBInfo:UNIT_STATS(event, unit)
	if (unit == "player") then
		PWBInfo:UpdateStats()
	end
end

function PWBInfo:SlowTimer()
	PWBInfo:Update(UnitPower("player"))
end

function PWBInfo:UpdateStats()
		-- local _, stamina, _, _ = UnitStat("player", 3); -- get stamina
		-- local _, strength, _, _ = UnitStat("player", 1); -- get strength
		-- local baseAP, posBuffAP, negBuffAP = UnitAttackPower("player"); --AP value

		-- local attackPowerSide = (baseAP + posBuffAP + negBuffAP - strength * 2) * 1.8;
		-- self.barrier = max(attackPowerSide, stamina * 2.5)

		-- WoD version
		local baseAP, posBuffAP, negBuffAP = UnitAttackPower("player")
		local attackPower = baseAP + posBuffAP + negBuffAP
		self.barrier = max(attackPower, 0) * 2.35
end

function PWBInfo:RefreshConfig()
	self:UpdateStyling()
	self:UpdateVisiblity()
end

function PWBInfo:Update(power)
	local shieldBarrierText = ""
	local resolveHealingText = ""
	local resolveDamageText = ""

	if self.db.shieldBarrier.enabled then
		if power > 19 then

			if power > 59 then
				shieldBarrierText = string.format("%.1fK", self.barrier / 1000)
			else
				shieldBarrierText = string.format("%.1fK", self.barrier * power / 60 / 1000)
			end

		end
	end

	if self.db.resolveHealing.enabled or self.db.resolveDamage.enabled then
		local n,_,_,_,_,_,_,_,_,_,id,_,_,val1,healing_bonus,damage_taken = UnitAura("player", self.resolve_name, nil, "HELPFUL");

		if n then

			if damage_taken > 0  and self.db.resolveDamage.enabled then
				resolveDamageText = string.format("%.1fK", damage_taken / 1000)
			end

			if healing_bonus > 0 and self.db.resolveHealing.enabled then
				resolveHealingText = healing_bonus .. "%"
			end

		end

	end
	
	ResolveHealingText:SetText(resolveHealingText)
	ResolveDamageText:SetText(resolveDamageText)
	ShieldText:SetText(shieldBarrierText)
end